import pika
import os

class DQuotNotificationProviderRabbitMQ:
    def __init__(self, host = '127.0.0.1', port = 5672, vhost = '/'):
        self.host = host
        self.port = port
        self.vhost = vhost
        self.connection = self.connection()

    def connection(self):
        credentials = pika.PlainCredentials(os.getenv('DQUOT_RABBITMQ_USER'),
                                            os.getenv('DQUOT_RABBITMQ_PASS'))
        parameters = pika.ConnectionParameters(self.host, self.port, self.vhost)
        return pika.BlockingConnection(parameters)

    def send(self, input):
        channel = self.connection.channel()
        channel.queue_declare(queue='quota.warnings')
        channel.basic_publish(exchange='helloEx', routing_key='', body='Hello World!')
        self.connection.close



import redis
import json
import os

class DQuotNotificationProviderRedis:
    def __init__(self, host = '127.0.0.1', port = 6379):
        self.redis = redis.StrictRedis(host=host,
                                       port=port,
                                       password=os.getenv('DQUOT_REDIS_PASS'))

    def send(self, input):
        notification = json.loads(input)
        self.redis.setex("quota:" + str(notification['uid']),
                         10, notification['message'])



from setuptools import setup
from setuptools import find_packages

setup(
    name = 'dquota',
    version = '0.1',
    author = 'Donatas Abraitis',
    author_email = 'donatas.abraitis@gmail.com',
    description = 'Handle quota notifications using generic netlink',
    url = 'https://github.com/ton31337/dquot-python',
    packages = ['dquota',
                'dquota.providers',
                'dquota.providers.stdout',
                'dquota.providers.redis',
                'dquota.providers.rabbitmq'
    ],
    install_requires = [
        'pika',
        'redis'
    ]
)




import time
import json
from pyroute2.netlink import genlmsg
from pyroute2.netlink.generic import GenericNetlinkSocket
from pyroute2.netlink.nlsocket import Marshal

QUOTA_NL_C_UNSPEC = 0
QUOTA_NL_C_WARNING = 1

class dquotmsg(genlmsg):
    prefix = 'QUOTA_NL_A_'
    nla_map = (('QUOTA_NL_A_UNSPEC', 'none'),
               ('QUOTA_NL_A_QTYPE', 'uint32'),
               ('QUOTA_NL_A_EXCESS_ID', 'uint64'),
               ('QUOTA_NL_A_WARNING', 'uint32'),
               ('QUOTA_NL_A_DEV_MAJOR', 'uint32'),
               ('QUOTA_NL_A_DEV_MINOR', 'uint32'),
               ('QUOTA_NL_A_CAUSED_ID', 'uint64'),
               ('QUOTA_NL_A_PAD', 'uint64'))


class MarshalDQuot(Marshal):
    msg_map = {QUOTA_NL_C_UNSPEC: dquotmsg,
               QUOTA_NL_C_WARNING: dquotmsg}


class DQuotSocket(GenericNetlinkSocket):
    def __init__(self):
        GenericNetlinkSocket.__init__(self)
        self.marshal = MarshalDQuot()
        if kernel[0] <= 2:
            self.bind(groups=0xffffff)
        else:
            self.bind()
        for group in self.mcast_groups:
            self.add_membership(group)

    def bind(self, groups=0, async=False):
        GenericNetlinkSocket.bind(self, 'VFS_DQUOT', dquotmsg,
                                  groups, None, async)


class DQuotNotifications:
    def __init__(self, provider):
        self.ds = DQuotSocket()
        self.provider = provider
        self.warnings = [
            "All good",
            "Inode hardlimit reached",
            "Inode grace time expired",
            "Inode softlimit reached",
            "Block hardlimit reached",
            "Block grace time expired",
            "Block softlimit reached",
            "Usage got below inode hardlimit",
            "Usage got below inode softlimit",
            "Usage got below block hardlimit",
            "Usage got below block softlimit"
        ]

    def run(self):
        self.ds.bind()
        self.ds.add_membership('events')
        while True:
            for msg in self.ds.get():
                self.provider.send(json.dumps({
                    'uid': msg.get_attr('QUOTA_NL_A_EXCESS_ID'),
                    'message': self.warnings[msg.get_attr('QUOTA_NL_A_WARNING')] }))
            time.sleep(0.1)

